--[[=============================================================================
    Commands received from the receiver proxy (ReceivedFromProxy)

    Copyright 2015 Control4 Corporation. All Rights Reserved.
===============================================================================]]

-- This macro is utilized to identify the version string of the driver template version used.
if (TEMPLATE_VERSION ~= nil) then
	TEMPLATE_VERSION.device_messages = "2015.03.31"
end

--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-- Power Functions
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
--[[
	Proxy Command: CONNECT_OUTPUT
	Parameters:
		output: mod 1000 value of Output Connection id
--]]
function CONNECT_OUTPUT(output)
    local command_delay = tonumber(Properties["Power On Delay Seconds"])
    local delay_units = "SECONDS"
    local command
    if (gControlMethod == "IR") then
		-- TODO: create packet/command to send to the device
		command = CMDS_IR_ZONES[output]["CONNECT_OUTPUT"]
		LogTrace("command = " .. command)
		PackAndQueueCommand("CONNECT_OUTPUT", command, command_delay, delay_units)
    else
		-- TODO: create packet/command to send to the device
		command = "POWERON" .. tPowerCommandMap[output]
		LogTrace("command = " .. command)
		PackAndQueueCommand("CONNECT_OUTPUT", command, command_delay, delay_units)
		
	     --set feedback verbosity. Feedback level 1: Data is sent whenever any of the statuses listed below changes. 
		PackAndQueueCommand("Set Feedback Verbosity to Level 1", "VERB(1)")
		
		
		GetDeviceVolumeStatus(output)
		
		-- TODO: If the device will automatically report power status after
		--	the On command is sent, then the line below can be commented out		
		GetDevicePowerStatus(output)
		
    end
end

--[[
	Proxy Command: DISCONNECT_OUTPUT
	Parameters:
		output: mod 1000 value of Output Connection id
--]]
function DISCONNECT_OUTPUT(output)
    local command_delay = tonumber(Properties["Power Off Delay Seconds"])
    local delay_units = "SECONDS"
    local command 
    
    if (gControlMethod == "IR") then
		-- TODO: create packet/command to send to the device
		command = CMDS_IR_ZONES[output]["DISCONNECT_OUTPUT"]
		LogTrace("command = " .. command)
		PackAndQueueCommand("DISCONNECT_OUTPUT", command, command_delay, delay_units)		
    else
		-- TODO: create packet/command to send to the device
		command = "POWEROFF" .. tPowerCommandMap[output]
		LogTrace("command = " .. command)
		PackAndQueueCommand("DISCONNECT_OUTPUT", command, command_delay, delay_units)
	
		-- TODO: If the device will automatically report power status after
		--	the Off command is sent, then the line below can be commented out
		GetDevicePowerStatus(output)		
    end  

end

--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-- Input Selection Functions
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
--[[
	Proxy Command: SET_INPUT
	Parameters:
		idBinding: proxybindingid of proxy bound to input connection
		output: mod 1000 value of Output Connection id	
		input: mod 1000 value of Input Connection id
--]]
function SET_INPUT(idBinding, output, input)
	local command
	
	if (idBinding == RECEIVER_PROXY_BINDINGID) then
		if (gControlMethod == "IR") then		
			command = tInputCommandMap_IR[output][tInputConnMapByID[input].Name]	
		else
		     if (output == 1) then
				if (tZoneBInputCommandMap[tInputConnMapByID[input].Name] == nil) then
				    LogWarn(tInputConnMapByID[input].Name .. " is not a valid Input for Zone B")
				    return
				end
			
				command = "ZSRC(" .. tZoneBInputCommandMap[tInputConnMapByID[input].Name] .. ")"
			else
				command = "SRC(" .. tInputCommandMap[tInputConnMapByID[input].Name] .. ")"
		     end
		end 		
	elseif (idBinding == TUNER_PROXY_BINDINGID) then
		if (tInputConnMapByID[input].Name == "FM Antenna") then 
		  gReceiverProxy._TunerBand = "FMBand" 
		elseif (tInputConnMapByID[input].Name == "AM Antenna") then 	
		  gReceiverProxy._TunerBand = "AMBand"
		else
		  LogWarn(tInputConnMapByID[input].Name .. " is not a valid Antenna Input for the AM/FM Tuner")
		  return
		end		
		
		if (gControlMethod == "IR") then		
			-- TODO: create packet/command to send to the device
			command = tInputCommandMap_IR[output][tInputConnMapByID[input].Name]
		else
			-- TODO: create packet/command to send to the device
			command =  tOutputCommandMap[output] .. tInputCommandMap[tInputConnMapByID[input].Name]
		end
	end
	LogTrace("command = " .. command)
	PackAndQueueCommand("SET_INPUT", command)
end

function INPUT_TOGGLE(idBinding, output)
	if (idBinding == TUNER_PROXY_BINDINGID) then
		local connection_name = ""
		if (gTunerBand == 'FMBand') then
			connection_name = "AM Antenna"
			gTunerBand = 'AMBand'
		else 
			connection_name = "FM Antenna"
			gTunerBand = 'FMBand'
		end
		
		if (gControlMethod == "IR") then			
			command = tInputCommandMap_IR[output][connection_name]
		else
			local command_value = tInputCommandMap[connection_name] 
			command = ""
		end		
		PackAndQueueCommand("INPUT_TOGGLE", command)
	end
end

--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-- Volume Control Functions
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
--[[
	Proxy Command: MUTE_OFF
	Parameters:
		output: mod 1000 value of Output Connection id	
--]]
function MUTE_OFF(output)
	local command
	if (gControlMethod == "IR") then		
		-- TODO: create packet/command to send to the device
		command = CMDS_IR_ZONES[output]["MUTE_OFF"]	
	else
		-- TODO: create packet/command to send to the device
		command = tMuteCommandMap[output] .. "OFF"
	end 	
	LogTrace("command = " .. command)
	PackAndQueueCommand("MUTE_OFF", command)
end

--[[
	Proxy Command: MUTE_ON
	Parameters:
		output: mod 1000 value of Output Connection id	
--]]
function MUTE_ON(output)
	local command
	if (gControlMethod == "IR") then			
		command = CMDS_IR_ZONES[output]["MUTE_ON"]	
	else
		-- TODO: create packet/command to send to the device
		command = tMuteCommandMap[output] .. "ON"
	end 	
	LogTrace("command = " .. command)
	PackAndQueueCommand("MUTE_ON", command)		
end

--[[
	Proxy Command: MUTE_TOGGLE
	Parameters:
		output: mod 1000 value of Output Connection id	
--]]
function MUTE_TOGGLE(output)
	local command
	if (gControlMethod == "IR") then			
		command = CMDS_IR_ZONES[output]["MUTE_TOGGLE"]	
	else
		-- TODO: create packet/command to send to the device
		command = tMuteCommandMap[output]
	end 
	LogTrace("command = " .. command)
	PackAndQueueCommand("MUTE_TOGGLE", command)
end

--[[
	Proxy Command: SET_VOLUME_LEVEL
	Parameters:
		output: mod 1000 value of Output Connection id	
		c4VolumeLevel: volume level to be set represented in Control4 scale (0-100)
--]]
function SET_VOLUME_LEVEL(output, c4VolumeLevel)
	-- TODO: edit "minDeviceLevel" & "maxDeviceLevel" values, based upon the protocol specification for volume range
	local minDeviceLevel = -999
	local maxDeviceLevel = 240
	local deviceVolumeLevel = ConvertVolumeToDevice(c4VolumeLevel, minDeviceLevel, maxDeviceLevel)	
	
	---- TODO: uncomment and edit string padding, if required, based upon the protocol specification
    --deviceVolumeLevel = string.rep("0", 2 - string.len(deviceVolumeLevel)) .. deviceVolumeLevel
	
	LogInfo('deviceVolumeLevel: ' .. deviceVolumeLevel)
  
	-- TODO: create packet/command to send to the device
	local command = tVolumeSetCommandMap[output] .. "(" .. deviceVolumeLevel .. ")"
	LogTrace("command = " .. command)
	PackAndQueueCommand("SET_VOLUME_LEVEL", command)
end

--[[
	Helper Function: SET_VOLUME_LEVEL_DEVICE
	Parameters:
		output: mod 1000 value of Output Connection id	
		deviceVolumeLevel: volume level to be set represented in device scale (as sepcified in the device's control protocol)
--]]
function SET_VOLUME_LEVEL_DEVICE(output, deviceVolumeLevel)
	--Called from ContinueVolumeRamping()
	
	-- TODO: create packet/command to send to the device
	local command = tVolumeSetCommandMap[output] .. "(" .. deviceVolumeLevel .. ")"
	
	LogTrace("command = " .. command)
	local command_delay = tonumber(Properties["Volume Ramp Delay Milliseconds"])
	PackAndQueueCommand("SET_VOLUME_LEVEL_DEVICE", command, command_delay)
end

--[[
	Proxy Command: PULSE_VOL_DOWN
	Parameters:
		output: mod 1000 value of Output Connection id	
--]]
function PULSE_VOL_DOWN(output)
	local command
	if (gControlMethod == "IR") then		
		-- TODO: create packet/command to send to the device
		command = CMDS_IR_ZONES[output]["VOLUME_DOWN"]	
	else
		-- TODO: create packet/command to send to the device
		command = tVolumeCommandMap[output] .. '-'
	end 		
	LogTrace("command = " .. command)
	local command_delay = tonumber(Properties["Volume Ramp Delay Milliseconds"])
	PackAndQueueCommand("PULSE_VOL_DOWN", command, command_delay)
end

--[[
	Proxy Command: PULSE_VOL_UP
	Parameters:
		output: mod 1000 value of Output Connection id	
--]]
function PULSE_VOL_UP(output)
	local command
	if (gControlMethod == "IR") then	
		-- TODO: create packet/command to send to the device
		command = CMDS_IR_ZONES[output]["VOLUME_UP"]	
	else
		-- TODO: create packet/command to send to the device
		command = tVolumeCommandMap[output] .. '+'
	end 		
	LogTrace("command = " .. command)
	local command_delay = tonumber(Properties["Volume Ramp Delay Milliseconds"])
	PackAndQueueCommand("PULSE_VOL_UP", command, command_delay)
end


function PULSE_BASS_DOWN()
    PackAndQueueCommand("PULSE_BASS_DOWN", "TRIMBASS-")
end

function PULSE_BASS_UP()
    PackAndQueueCommand("PULSE_BASS_UP", "TRIMBASS+")
end

function SET_BASS_LEVEL(output, c4BassLevel)
    PackAndQueueCommand("SET_BASS_LEVEL", "TRIMBASS(" .. c4BassLevel .. ")")
end

function PULSE_TREBLE_DOWN()
    PackAndQueueCommand("PULSE_TREBLE_DOWN", "TRIMTREB-")
end

function PULSE_TREBLE_UP()
    PackAndQueueCommand("PULSE_TREBLE_UP", "TRIMTREB+")
end

function SET_TREBLE_LEVEL(output, c4BassLevel)
    PackAndQueueCommand("SET_TREBLE_LEVEL", "TRIMTREB(" .. c4BassLevel .. ")")
end

--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-- Tuner Proxy Functions
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

function SET_CHANNEL(idBinding, output, band, channel)
	if (band == "FM") then
		_, _, sMajor =string.find(channel,'(.-)%D')
		_, _, sMinor =string.find(channel,'%D(.*)')
		if (sMajor == nil) or (sMinor == nil) then return end
		--INT'L TUNING: depending on tFreqStepInfo[gFreqStep].FmSpacing, 'frequency' may need to be adjusted for proper command syntax
		channel = string.format('%03d', sMajor) .. sMinor .. string.rep("0", 2 - string.len(sMinor))
	elseif (band == "AM") then
	     --INT'L TUNING: depending on tFreqStepInfo[gFreqStep].AmSpacing, 'frequency' may need to be adjusted for proper command syntax
		channel = string.rep("0", 5 - string.len(channel)) .. channel
	end 

	if (gControlMethod == "IR") then	
		-- TODO: create packet/command to send to the device (if a Direct Tuner command is needed, map it to a remote button like "STAR")
		--SEND_COMMAND_FROM_COMMAND_TABLE(idBinding, output, "STAR")
		for i=1,string.len(channel) do
			SEND_COMMAND_FROM_COMMAND_TABLE(idBinding, output, "NUMBER_" .. string.sub(channel,i,i))
		end
	else
		-- TODO: create packet/command to send to the device
		local command = tTunerTuningCommandMap[output] .. channel
		LogTrace("command = " .. command)
		PackAndQueueCommand("SET_CHANNEL", command)		
	end
end

function SET_PRESET(idBinding, output, preset)
	if (gControlMethod == "IR") then	
		for i=1,string.len(preset) do
			SEND_COMMAND_FROM_COMMAND_TABLE(idBinding, output, "NUMBER_" .. string.sub(preset,i,i))
		end
	else
		-- TODO: create packet/command to send to the device
		local command = tTunerPresetCommandMap[output] .. preset
		LogTrace("command = " .. command)
		PackAndQueueCommand("SET_PRESET", command)			
	end	
end

--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-- Surround Mode Function
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
function SET_SURROUND_MODE(mode)
	-- TODO: create packet/command to send to the device
	local command = tSurroundModes[mode]
	LogTrace("command = " .. command)
	PackAndQueueCommand("SET_SURROUND_MODE", command)
end

--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-- Helper Functions
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

--[[
	Helper Function: SEND_COMMAND_FROM_COMMAND_TABLE
	Parameters:
		idBinding: proxy id	
		output: mod 1000 value of Output Connection id
		command_name: name of command to be sent
--]]
function SEND_COMMAND_FROM_COMMAND_TABLE(idBinding, output, command_name)
    local output_for_log = output or "nil"
    LogTrace("SEND_COMMAND_FROM_COMMAND_TABLE(), idBinding=" .. idBinding .. ", output=" .. output_for_log .. ", command_name=" .. command_name)
	
	-- TODO: create packet/command to send to the device
	local command = GetCommandFromCommandTable(idBinding, output, command_name)
	
	if (command == nil) then
		LogTrace("command is nil")
	else
		LogTrace("command = " .. command)
		PackAndQueueCommand(command_name, command)
	end			
end

--[[
	Helper Function: GetCommandFromCommandTable
	Parameters:
		idBinding: proxy id	
		output: mod 1000 value of Output Connection id
		command_name: name of command to be returned
--]]
function GetCommandFromCommandTable(idBinding, output, command_name)
	LogTrace("GetCommand()")
	local t = {}
	
	-- TODO: create appropriate commands table structure
	
	if (gControlMethod == "IR") then
		t = CMDS_IR
	else
		t = CMDS
	end	

	if (t[idBinding] ~= nil) then
	   if (t[idBinding][command_name] ~= nil) then
		  return t[idBinding][command_name]
	   end
	end
	
	if (t[command_name] ~= nil) then
		return t[command_name]
	else
		LogWarn('GetCommandFromCommandTable: command not defined - '.. command_name)
		return nil
	end	
	
--[[	
	--this code sample uses the "output" parameter for indexing the IR command table
	--not all proxies return an output parameter so this must be derived using the findOutputFromBindingID() and related functions
	if (gControlMethod == "IR") then
		t = CMDS_IR_ZONES
		if (output == nil) then
			output = findOutputFromBindingID(idBinding)
		end	
		if (t[output][strCommandName] ~= nil) then
			return t[output][strCommandName]
		elseif (t[strCommandName] ~= nil) then
			return t[strCommandName]
		else
			LogWarn('GetCommand(): Command not defined - '.. strCommandName)
			return nil
		end			
	else
		t = CMDS

		if (t[idBinding][strCommandName] ~= nil) then
			return t[idBinding][strCommandName]
		elseif (t[strCommandName] ~= nil) then
			return t[strCommandName]
		else
			LogWarn('GetCommand(): Command not defined - '.. strCommandName)
			return nil
		end		
		
	end		
--]]
end

--[[
	Helper Function: findOutputFromBindingID
	Parameters:
		idBinding: proxy id	
	Called from GetCommandFromCommandTable() when the deviceprotocol has multi-zone 
	command structures for commands which Control4 does not include an OUTPUT parameter
--]]
function findOutputFromBindingID(idBinding)
	-- TODO: edit tProxyInputNameMapByProxyID based upon the AV proxy connection modelling
	tProxyInputNameMapByProxyID = {
		[TUNER_PROXY_BINDINGID] = "AM/FM TUNER", 
		
	}
	
	LogTrace('findOutputFromBindingID: ' .. idBinding)
	local name = tProxyInputNameMapByProxyID[idBinding] 
	local output = findOutputIDFromInputName(name)
	return output
end

--[[
	Helper Function: findOutputIDFromInputName
	Parameters:
		inputName: input connection  name (this will be the input for the AV proxy connection)	
	Called from findOutputFromBindingID() when the device protocol has multi-zone 
	command structures for commands which Control4 does not include an OUTPUT parameter
--]]
function findOutputIDFromInputName(inputName)
    LogTrace('findOutputIDFromInputName: ' .. inputName)
    local output
    for j,k in pairs(gReceiverProxy._CurrentlySelectedInput) do
		if (tInputConnMapByID[k] ~= nil) then
			if (inputName==tInputConnMapByID[k].Name) then
				if (j==0) then 
					return j 
				else
					output = j
				end
			end	
		end
	end
    return output 
end

--[[
	Helper Function: GetDeviceVolumeStatus
--]]
function GetDeviceVolumeStatus(output)
    LOG:Trace("GetDeviceVolumeStatus(), output = " .. output)
	
	-- TODO: verify table entries in tVolumeQueryMap for all zones
	local command = tVolumeQueryMap[output]  .. "?"
	LOG:Trace("command = " .. command)
	PackAndQueueCommand("GetDeviceVolumeStatus: Volume", command)
	
	-- TODO: verify table entries in tMuteCommandMap for all zones, modify line below if needed
	command = tMuteCommandMap[output] .. "?"
	LOG:Trace("command = " .. command)
	PackAndQueueCommand("GetDeviceVolumeStatus: Mute", command)	
end

--[[
	Helper Function: GetDevicePowerStatus
--]]
function GetDevicePowerStatus(output)
    LOG:Trace("GetDevicePowerStatus()")
	
	local command = tPowerQueryMap[output] .. "?"
	LOG:Trace("command = " .. command)
	PackAndQueueCommand("GetDevicePowerStatus: Volume", command)	
end
